float get_battery_voltage(void){  //~

   float voltage;
   int16 value, aa,n;
   int32 x;
//   setup_adc_ports(AN0_AN1_AN3);
   setup_adc_ports(AN0);
   setup_adc(ADC_CLOCK_DIV_64);
   set_adc_channel( 0 );
   // take an average of 30 readings
   x=0;
   for(n=0; n<30; n++){
      aa = read_adc();
      x = x+aa;
   }
   x = x/30;

   voltage = ((float) x) / 3280;  //  *4(v. divider) *5V(max)

   return voltage;
}

//-----------------------------------------------------------------------
void lcd_move_cursor(int8 horiz, int8 vert){

   int8 aa;

   if((horiz < 1) || (horiz > 20) || (vert < 1) || (vert > 2)){
      LCD_CLEAR
      printf(lcd_putc, "cursor error  ");
      while(1);
   }
   aa = horiz + 0x7F;
   if(vert != 1) {aa += 0x40;}
   lcd_putc(0x03);      // control c makes the next byte an LCD command
   lcd_putc(aa);         // cursor position code

}


//-------------------------------------------
//
// return error code, gpsstring[]
//

int8 get_gpsstring(void){  //~

   int8 n, max = 50;
   int32 time_to_give_up;

   time_to_give_up = current_time + 2000;

      // wait for '$', but don't wait TOO long
      while(1){
         if(current_time >= time_to_give_up) return ERROR; //added sept26,2008
         n = gps_getc();
         if(n == '$') break;  // found beginning of NMEA sentance
      }

//putc('&');
      // fill the array starting with '$'
      gpsstring[0] = '$';
      for(n=1; n<max; n++){
         gpsstring[n] = gps_getc();
         if(gpsstring[n] == '\r' || gpsstring[n] == '\n'){
            gpsstring[n] = '\0';
            return SUCCESS;
         }
      }

      // if string is too long...
      gpsstring[n] = '\0';//putc('#');
//      return ERROR;
      return SUCCESS;
}

//-----------------------------------------------------------------------

void steer(int16 position){  //~

  steer_timer_value = 65536 - (position * 10 / 4);

}

//-----------------------------------------------------------------------

void aim_sensor(int16 position){  //~

   sensor_servo_timer_value = 65536 - (position * 10 / 4);

}

//-----------------------------------------------------------------------


void tone(int16 frequency, int16 duration)
{

   int16 pulse, ticks;
   int32 end_time;

   end_time = current_time + duration;
   pulse = 500000 / frequency;

   while(current_time < end_time){

      output_low(PIEZO_PIN);
      delay_us(pulse);

      output_high(PIEZO_PIN);
      delay_us(pulse);

   }
}


//-----------------------------------------------------------------------


int16 sonar(void){  //~

   int16 ticks, time, distance;
   //disable_interrupts(GLOBAL);                   //debug
   output_low(SONAR_TRIGGER_PIN);
   if(input(SONAR_ECHO_PIN) == 1){
     // LCD_CLEAR sprintf(string, "no response from sonar\0"); lcd_send_string(string);
     delay_ms(2000);
     return(0);
   }

   output_high(SONAR_TRIGGER_PIN);                     // send ping
   delay_us(12);         // delay_ms over 10us
   output_low(SONAR_TRIGGER_PIN);
   while (input(SONAR_ECHO_PIN) == 0);           // wait for pulse

   // start timer
   setup_timer_3(T3_INTERNAL|T3_DIV_BY_8);
   set_timer3(0);

   while (input(SONAR_ECHO_PIN) == 1);           // wait for end of pulse
   enable_interrupts(GLOBAL);               //debug
   ticks = get_timer3();
   time = ticks / MHZ * 4 * 8 ;

/*
to convert timer results to cm, divide time by 2 (signal goes back and forth)
then multiply by 345/10000 (which is 0.0345 cm per us).
note: Speed of sound is 345m/s
*/
//   distance = time/2*345/10000;

   distance = time / 58;
   delay_ms(10); // time between measurements

   return distance;

}

//-----------------------------------------------------------------------

int16 compass(void)  //~
{
// 1-37ms = 0-360 degrees

  int16 degreesx;
  int16 ticks, time;

  while (input(COMPASS_PULSE_PIN) == 1); // if in the middle of a pulse, wait
  while (input(COMPASS_PULSE_PIN) == 0); // wait for pulse to start

  setup_timer_3(T3_INTERNAL|T3_DIV_BY_8);  set_timer3(0);
 // while (get_timer3() < ticks);

  while (input(COMPASS_PULSE_PIN) == 1);      // wait for end of pulse

// time in microseconds = ticks * 4 * prescaler / MHZ

  time = get_timer3() / MHZ * 4 * 8;

// degrees = (time in milliseconds - 1) * 10
   degreesx = (time / 100) - 10;

   return degreesx;

} // end of compass


//-----------------------------------------------------------------------


void lcd_putc(char aa){
   // 9600 baud = 104.1us/bit

   char n, b;

   // send start bit
   output_low(LCD_PIN);
   delay_us(104);

   // send 8 data bits
   for(n=1; n<=8; n++){
      b = aa & 0x01;             // b is the rightmost bit of a
      if (b == 1) {output_high(LCD_PIN);}
      else {output_low(LCD_PIN);}
      delay_us(104);
      aa >>= 1;                  // next bit
   }  // end for loop

   // send stop bit
   output_high(LCD_PIN);
   delay_us(104);
   delay_ms(2);

}

//-----------------------------------------------------------------------
void omi(char aa){
   // 9600 baud = 104.1us/bit

   char n, b;

   // send start bit
   output_low(OMI_PIN);
   delay_us(104);

   // send 8 data bits
   for(n=1; n<=8; n++){
      b = aa & 0x01;             // b is the rightmost bit of a
      if (b == 1) {output_high(OMI_PIN);}
      else {output_low(OMI_PIN);}
      delay_us(104);
      aa >>= 1;                  // next bit
   }  // end for loop

   // send stop bit
   output_high(OMI_PIN);
   delay_us(104);
   delay_ms(2);

}

//

void gets_with_echo(void){

   int8 max = 50;
   int16 n;

   // fill the array
   for(n=0; n<max; n++){
      str[n] = getc();
      putc(str[n]);   //  echo
      if(str[n] == '\r' || str[n] == '\n'){
         str[n] = '\0';
         return;
      }
   }

   // if string is too long...
   gpsstring[n] = '\0';
   return;

}



//--------------------------------------------------------

void aim_sonar ( int16 position, int16 num ){

// example: to move the sensors a short distance to far left,
// use "aim_sonar ( 1100, 6 );"
// 25 pulses is enough to get the servo anywhere

// crystal: 10MHz
// multiplyer:4
// clock: 10 x 4 = 40MHz
// system: 40 / 4 = 10MHz
// timer 0 prescaler: 4
// timer 0 speed: 10/4 = 2.5MHz

   sensor_servo_timer_value = 0 - (position * 10/4);
   //delay_ms(num*40);     //debug shows problem with this calculation.  The WHY is unknown.
   delay_ms(200);
}

//--------------------------------------------------------

void five_point_scan (void)
{

char n;
int16 pulse_width[6] = {0,2400,2000,1600,1200,800}; // pulse[0] isn't used
char offset[6]= {0,15,30,30,30,15}; // distance to edge of robot

   //  zone    direction        pulse width
   //  ----  ----------------   ----------
   //   1    60 degrees left    2100  2400
   //   2    30 degrees left    1900  2000
   //   3    straight ahead     1600  1600
   //   4    30 degrees right   1300  1200
   //   5    60 degrees right   1100  800

   lcd_move_cursor(1,2);
   aim_sonar (pulse_width[1],25);

   for(n = 1; n <= 5; n++){
      aim_sonar (pulse_width[n],8);
      zone[n] = sonar();
      if(zone[n] > offset[n]){zone[n] -= offset[n];}
      else{zone[n] = 0;}
      printf(lcd_putc,"%u ",zone[n]);
   }
   aim_sonar (pulse_width[3],25);
   //delay_ms(1000);
}

//-------------------------------------------------------

int16 look_left(void){
   aim_sonar (SONAR_LEFT,25);
   return sonar();
}

int16 look_ahead(void){
   aim_sonar (SONAR_AHEAD,25);
   return sonar();
}

int16 look_right(void){
   aim_sonar (SONAR_RIGHT,25);
   return sonar();
}

//--------------------------------------------------------

void blink(int16 nn){

  int16 n;

  for(n=0;n<nn;n++){
    output_low(STATUS_LED);
    delay_ms(200);
    output_high(STATUS_LED);
    delay_ms(200);
  }
}
//--------------------------------------------------------

void tx_info (void){


      putc('x'-CTRL);   //clear screen
      delay_ms(500);

      printf("state: %u  \r", state);
      delay_ms(100);

      printf("leg: %u \r", current_leg);
      delay_ms(100);

      printf("ch: %4Lu deg \r", compass() );
      delay_ms(100);

      printf("dh: %4Lu deg \r", desired_heading);
      delay_ms(100);

      printf("dr: %Lu m\r", (distance_remaining_since_gps_check - (wheel_ticks_since_gps_check / 105)));
      delay_ms(100);

      printf("sonar: %Lu cm\r", sonar());

      delay_ms(100);

      return;
}


//--------------------

void motor_speed (unsigned char aa){//~

   // 9600 baud = 104.1us/bit

   char n, b;

   // send start bit
   output_low(MOTOR_PIN);
   delay_us(104);

   // send 8 data bits
   for(n=1; n<=8; n++){
      b = aa & 0x01;             // b is the rightmost bit of a
      if (b == 1) {output_high(MOTOR_PIN);}
      else {output_low(MOTOR_PIN);}
      delay_us(104);
      aa >>= 1;                  // next bit
   }  // end for loop

   // send stop bit
   output_high(MOTOR_PIN);
   delay_us(104);

}  // end function


//-----------------------------------------------------------------------

float getfloat(void){

   int8 n, max = 10;
   int8 s[11];

   while(1){
      for(n=0; n<max; n++){
         s[n] = getc();
         if(s[n] == '\n'){
            s[n] = '\0';
            return atof(s);
         }
         putc(s[n]);   // echo
      }
      printf("error, try again");
   }
}


//--------------------------------------------------------

int8 gps_getc(void){  //~

   // 9600 baud = 104.1us/bit

   char n, b=0;
   int32 time_to_give_up;

   time_to_give_up = current_time + 200;

   // wait for start bit, but don't wait TOO long
   while(TRUE){
      if(input(GPS_PIN) == 0) break;
      if(current_time >= time_to_give_up) return ERROR; //added sept26,2008
   }

   delay_us(52);
   if (input(GPS_PIN) == 1) {return ERROR;}

   // get 8 data bits
   for(n=1; n<=8; n++){
      delay_us(104);
      b >>= 1;                  // next bit
      if (input(GPS_PIN) == 1) bit_set(b,7);
   }  // end for loop

   // stop bit
   delay_us(104);

   if (input(GPS_PIN) == 0) {return ERROR;}

   return b;
}
