
#include <robomagellan2.h>  //  defines, includes, function list, global variables
#include <devices.h>

#include <rm_states.c>      //  the behavior
#include <interrupts.c>     //  heartbeat, wheel, servos, clock
#include <devices.c>
//-----------------------------------------------------

void main(){

   signed int8 temp;
   signed int8 j, e;
   int16 i;  //  used in for loops
   int32 start_time;

   // initialization
   port_b_pullups(TRUE);

   output_high(LCD_PIN);               // software UART
   output_high(MOTOR_PIN);             // software UART

   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_4); //resolution is .4 us, servos
   enable_interrupts(INT_TIMER0);
   setup_timer_2(T2_DIV_BY_16,39,16);  // system clock, 1ms
   enable_interrupts(INT_TIMER2);
   //   setup_timer_1(T1_EXTERNAL|T1_DIV_BY_1);  // for front wheel
   enable_interrupts(GLOBAL);

   delay_ms(2000);

   // check battery voltage
   if(get_battery_voltage() < 12.0){
      LCD_CLEAR
      printf(lcd_putc, "%g", get_battery_voltage());
      delay_ms(2000);
   }

   LCD_CLEAR
   printf(lcd_putc,"RoboMagellan");
   delay_ms(700);

   // get waypoints from EEPROM
   j = NUMBER_OF_WAYPOINTS * 4;
   for(i=0;i<j;i++){
      *(&lat[0]+i) = read_eeprom(i);
   }
      e =  NUMBER_OF_WAYPOINTS * 4;
   for(i=0;i<j;i++){
      *(&lon[0]+i) = read_eeprom(i+e);
   }

   state = _STANDBY;
   while(1){
      if (state==_STANDBY) standby();
      if (state==_BEGINNING) beginning();
      if (state==_MIDDLE) middle();
      if (state==_ENDING) ending();
      if (state==_OBSTACLE) obstacle();
      if (state==_QUIT) quit();
   }

}

//-----------------------------------------------------------------------
//
// RETURNS SPEED IN CM/S
//
int16 get_speed(void){

  int16 speed, distance;

  distance = (wheel_count_recent - wheel_count_old); // note: 1 count =  1 cm
  speed = distance * (1000/SPEED_UPDATE_INTERVAL);

  return speed;

}
//-----------------------------------------------------------------------
//
//GO A DISTANCE IN METERS
//

int1 go_meters(signed int16 meters){

   int16 distance, distance_travelled, distance_to_travel;
   int32 start_time, start_distance, end_distance;
   int32 time_to_tx_info;

   distance_to_travel = ABS ((int16)meters) * 100;
   start_distance = wheel_count;
   if (meters<0){
      motor_speed(-60);
   }
   else
      motor_speed(60);
   start_time = current_time;

   while(1){
      distance_travelled = (int16)(wheel_count - start_distance);
      if (distance_travelled >= distance_to_travel) return TRUE;   //go beginning

      // check distance
      distance = sonar();

     //too close?
     if((meters > 0) && (distance < 30 || input(RIGHT_BUMPER_PIN) ||input(LEFT_BUMPER_PIN ))){                       //debug
        return FALSE;
     }

     //send info to hhc (handheld control) every 2 seconds
     if(current_time >= time_to_tx_info){
        tx_info();
        time_to_tx_info = current_time + 2000;
     }

     /* //timeout?
     if(current_time > start_time + 10000){
        //beep
        tone(500,1000);
        tone(1000,1000);
     }

     //stopped?
     if(get_speed() < 2 ){
       //beep
       tone(3000,1000);
       motor_speed(0);
     }*/

  } // end of main loop

} // end of go_meters function

//-----------------------------------------------------------------------

void handle_user_input(void){   //~

   float x1,y1,x2,y2;
   int16 x,y;
   int8 n, i, e;
   int16 temp16;
   float value;
   int16 waynum;
   int16 distance;

   // PROMPT USER
   printf(">");

   if(OERR){
     for(n=1; n<=5; n++){
        output_low(STATUS_LED);
        delay_ms(50);
        output_high(STATUS_LED);
        delay_ms(50);
     }
     CREN=0;
     CREN=1;
   }

   input=getchar();

   // DISPLAY WAYPOINTS ?
   if (input=='w'){
      putc('x'-CTRL); delay_ms(GLCD_CLEAR_DELAY);
      printf("#  Lat.   Long.\r" );
      for(n=1; n<=5; n++){
         y = (int16)(( lat[n] - lat[1] ) * Y_FACTOR);
         x = (int16)(( lon[n] - lon[1] ) * X_FACTOR);
           printf("%d%7.3g %7.3g\r", n, lat[n],lon[n]);
         delay_ms(150);
      }
      printf("\n\r");
   }

   // WAYPOINT EDITING?
   else if (input=='W'){
      putc('x'-CTRL); delay_ms(GLCD_CLEAR_DELAY);

      printf("which waypoint? ");
      waynum = getc();   // echo
      putc(waynum);
      waynum -= '0';     // convert to ASCII
      putc(13);

      printf("lat: ");
      gets_with_echo();
      value = atof(str);
      lat[waynum] = value;

      printf("lon: ");
      gets_with_echo();
      value = atof(str);
      if(value > 0) value = 0 - value;
      lon[waynum] = value;

      // store in eeprom
      e = NUMBER_OF_WAYPOINTS * 4;
      for(i=0; i<4; i++){
         write_eeprom((waynum*4)+i, *(&lat[waynum]+i));
         write_eeprom((waynum*4)+i+e, *(&lon[waynum]+i));
      }

   }

   //show route?
   else if (input=='r'){
      putc('x'-CTRL); delay_ms(GLCD_CLEAR_DELAY);

   for(n=1; n<ROUTE_MAX; n++){
      if(route[n] == 0) break;
      printf("%u  ", route[n]);
   }

      printf("\r\n");
   }

   //edit route
   else if (input=='R'){
      putc('x'-CTRL); delay_ms(GLCD_CLEAR_DELAY);

       printf("enter route:");

      for(n=1; n<ROUTE_MAX; n++){
         route[n] = getc();
         if(route[n] > '0' && route[n] <= '9'){  // if a number..
            putc(route[n]);   // echo
            route[n] -= '0';
         }
         if(route[n] == '\r'){
            route[n] = '\0';
            printf("\r\n");
            return;
         }
      }
      printf("error, try again");
   }

   // distances
      else if (input=='d'){
         putc('x'-CTRL); delay_ms(GLCD_CLEAR_DELAY);  // clear screen
         printf("waiting for GPS....");
         while(1){
            n=get_gpsstring();
            //if(n==ERROR){putc('E');break;}
            if(gpsstring_to_minutes() == SUCCESS) break;
         }

         printf("# deg  meters\r");    delay_ms(150);

         for(waynum=1;waynum<=5;waynum++){
            x = (signed int16)((lon[waynum] - current_lon) * X_FACTOR);
            y = (signed int16)((lat[waynum] - current_lat) * Y_FACTOR);
            desired_heading = xy_to_heading(x,y);
            distance_remaining_since_gps_check = xy_to_distance(x,y);
            if(distance_remaining_since_gps_check > 999){
               distance_remaining_since_gps_check = 999;
            }

    //        printf("%Lu %3Lu %Lu  %Ld %Ld\r", waynum ,desired_heading, distance_remaining_since_gps_check,x,y);               delay_ms(150);
            printf("%Lu %3Lu %Lu \r", waynum ,desired_heading, distance_remaining_since_gps_check);               delay_ms(150);
         }
      }

   //sensors

   else if (input=='s'){
      putc('x'-CTRL); delay_ms(GLCD_CLEAR_DELAY);

   // printf("%Lu sec\r", current_time/1000 );             delay_ms(150);
      printf("Comp: %Lu deg\r", compass() );               delay_ms(GLCD_LINE_DELAY);
      printf("%Lu ticks\r", wheel_count );                 delay_ms(GLCD_LINE_DELAY);
      printf("Bat:%6.2f volts\r", get_battery_voltage());  delay_ms(GLCD_LINE_DELAY);
      printf("Sonar: %Lu cm\r", sonar());                  delay_ms(GLCD_LINE_DELAY);

      while(1){
         n = get_gpsstring();
         if(n == ERROR){
            printf("no GPS data\r");
            break;
         }
         if(gpsstring_to_minutes() == SUCCESS){
            printf("%7.3g %7.3g\r", current_lat, current_lon);
            break;
         }
      }

      if (input(LEFT_BUMPER_PIN)){
         putc('L');
      }
      if (input(RIGHT_BUMPER_PIN)){
         putc('R');
      }
/*
      printf("\r");
         putc(0x1b);
         putc('r');
         putc(0x1b);
         putc('f');delay_ms(500);
         putc(0x1b);
         putc('k');
         putc(128+(4*16)+5);

      printf("WARNING");                  delay_ms(GLCD_LINE_DELAY);
      putc(128+(6*16)+3);
      printf("LOW BATTERY");                  delay_ms(GLCD_LINE_DELAY);
*/

   }

   // START?

   else if (input=='g'){
      putc('x'-CTRL); delay_ms(GLCD_CLEAR_DELAY);
      current_leg = 1;
      state=_BEGINNING;
      return;
   }

   // STORE CURRENT POSITION?

   else if (input=='c'){
      putc('x'-CTRL); delay_ms(GLCD_CLEAR_DELAY);

      // get waynum from user
      printf("waypoint #:");

      waynum = getc();
      putc(waynum);       // echo
      waynum -= '0';      // convert to ASCII

      if((waynum < 1) || (waynum > 5)) {
         printf("error: %Lu", waynum);
         delay_ms(2000);
         continue;
      }

   // get lat and lon from GPS reciever
      printf("waiting for GPS\r");
      while(1){
         get_gpsstring();
         if(gpsstring_to_minutes() == SUCCESS){          //
            break;
         }
      }

      // store in RAM
      lat[waynum] = current_lat;
      lon[waynum] = current_lon;

      // store in EEPROM
      e = NUMBER_OF_WAYPOINTS * 4;
      for(i=0; i<4; i++){
         write_eeprom((waynum*4)+i, *(&lat[waynum]+i));
         write_eeprom((waynum*4)+i+e, *(&lon[waynum]+i));
      }

      printf("waypoint %Ld stored", waynum);

   }
// FORWARD
   else if (input=='9'){
      steer(STEER_STRAIGHT);
      motor_speed(80);
   }

// FAST FORWARD
   else if (input==','){
      steer(STEER_STRAIGHT);
      motor_speed(100);
   }

// STOP
   else if (input=='o'){
      motor_speed(0);
   }

// REVERSE
   else if (input=='l'){
      steer(STEER_STRAIGHT);
      motor_speed(-80);
   }

// HARD LEFT
   else if (input=='i'){
      steer(STEER_HARD_LEFT);
      motor_speed(80);
   }

// HARD RIGHT
   else if (input=='p'){
      steer(STEER_HARD_RIGHT);
      motor_speed(80);
   }

// SLIGHT LEFT
   else if (input=='8'){
      steer(STEER_SLIGHT_LEFT);
      motor_speed(80);
   }

// SLIGHT RIGHT
   else if (input=='0'){
      steer(STEER_SLIGHT_RIGHT);
      motor_speed(80);
   }

// BEEP
   else if (input=='.'){
      tone(710,200);
   }

// brighter
   else if (input=='b'){
   putc('q'-CTRL);
   }

// darker
   else if (input=='B'){
   putc('a'-CTRL);
   }

   //main menu?
   else if (input=='?'|| input=='h'){
      putc('x'-CTRL); delay_ms(GLCD_CLEAR_DELAY);

      printf(" - MAIN MENU -\r");
//      printf("\r");
      printf("w show waypoints\r");         delay_ms(GLCD_LINE_DELAY);
      printf("W edit waypoints\r");         delay_ms(GLCD_LINE_DELAY);
      printf("r show route\r");             delay_ms(GLCD_LINE_DELAY);
      printf("R edit route\r");             delay_ms(GLCD_LINE_DELAY);
      printf("d distances\r");              delay_ms(GLCD_LINE_DELAY);
      printf("s sensor values\r");          delay_ms(GLCD_LINE_DELAY);
      printf("g start\r");                  delay_ms(GLCD_LINE_DELAY);
      printf("c store cur pos\r");          delay_ms(GLCD_LINE_DELAY);
      printf("b brighter\r");               delay_ms(GLCD_LINE_DELAY);
      printf("B darker\r");                 delay_ms(GLCD_LINE_DELAY);
   }

   // invalid option?
   else printf("invalid option(%2X)\r", input);
   return;

} // end of handle_user_input

//-----------------------------------------------------

//-----------------------------------------------------
//
//  input:    x (a number that increases going east)
//  input:    y (a number that increases going north)
//
//  returns:  heading in degrees
//-----------------------------------------------------

int16 xy_to_heading(signed int16 x, signed int16 y){

   int8 n;
   int16 heading;
   float value;

//   float z,
//   float val[18]= {
//0.0, 0.09, 0.18, 0.27, 0.36, 0.47, 0.58, 0.70, 0.84,
// 1.0, 1.19, 1.43, 1.73, 2.14, 2.75, 3.73, 5.67, 11.43
//} ;

   if(x==0) x = 1;  // prevent a divide by zero error (a value of 1 is precise enough given the accuracy of GPS)


   value =  (atan_degree ( ABS((float)y) / ABS((float)x) ));

   /*if(x==0) x = 1;  // prevent a divide by zero error
   //if(y==0) y = 1;  // prevent a divide by zero error

   value = ( ABS(y) / ABS(x) );
   value =  (atan_degree );*/

   if(y>=0 AND x>=0) heading = 90 - (int16)value;   // quadrant 1 (north & east)
   else if(y<0 AND x>=0) heading = 90 + (int16)value;   // quadrant 2 (south & east)
   else if(y<0 AND x<0) heading = 270 - (int16)value;   // quadrant 3 (south & west)
   else if(y>=0 AND x<0) heading = 270 + (int16)value;   // quadrant 4 (north & west)

  // heading = (int16)value;
   return heading;
}

//------------------------------------------------------------------------
//
int16 xy_to_distance(signed int16 x, signed int16 y){

   int16 distance;

   distance = (int16)(sqrtb((x*x)+(y*y)));

   return distance;

}

//-----------------------------------------------------------------------

void deadstop(void){
   // code needed here
   printf("stopped!\n\r\n\r");

}

//-----------------------------------------------------------------------
// input: gpsstring (global)
// returns: current_lat, current_lon (global), error code
//
// example data string: $GPGLL,4523.630,N,07542.202,W,072248,A*35
//                               ^^^^^^      ^^^^^^
// extract minutes
// lat = 23.630
// lon = -42.202  (lon is always negative)

int8 gpsstring_to_minutes(void){  //~

   char tempstr[16];

   // check if right "NMEA sentance"
   strcpy(tempstr,"$GPGLL");
   if(strncmp(gpsstring,tempstr,6)!=0) return ERROR;

   // check if lan & lon in "sentance"
   strcpy(tempstr,"$GPGLL,,");
   if(strncmp(gpsstring,tempstr,8)==0) return ERROR;

   // extract lat minutes
   memcpy(str,&gpsstring[9],6);
   str[6]=0;
   current_lat=atof(str);

   // extract lon minutes
   memcpy(str,&gpsstring[21],6);
   str[6]=0;
   current_lon = 0 - atof(str);  // (always negative)

   return SUCCESS;
}

//------------

////////////////////////////////////////////////////////////////////////////
//   float sqrt(float x)
////////////////////////////////////////////////////////////////////////////
// Description : returns the square root of x
// Date : N/A
//
float sqrtb(float x)
{
   float y, res;
   BYTE *p;

   if( x<=0.0)
      return(0.0);

   y=x;
   p=&y;
   (*p)=(BYTE)((((int16)(*p)) + 127) >> 1);

   do {
      res=y;
      y+=(x/y);
      (*p)--;
   } while(res != y);

   return(res);
}

//--------------------------------------------------------

float atan_degree(float z){

   int8 n;
   int16 heading;
   float val[18]= {
   0.0, 0.09, 0.18, 0.27, 0.36, 0.47, 0.58, 0.70, 0.84,
   1.0, 1.19, 1.43, 1.73, 2.14, 2.75, 3.73, 5.67, 11.43
   } ;

   for(n=0;n<17;n++){
      if( z <= val[n]) break;
   }

   return n * 5;

}


