//  Robomagellan2
//  last revised: Apr. 29, 2010

#include <18F452.h>
#device adc=16  //  this line comes before the other includes

#include <stdio.h>
#include <stdlib.h>

#use delay(clock=40000000)
#fuses H4, BORV20, PUT, NOLVP, NOWDT  //(for 18F452)
//#fuses H4,  PUT, NOLVP, NOWDT           //(for 18F4520)
#use rs232(baud=9600,parity=N,xmit=PIN_C6,rcv=PIN_C7,bits=8)

#bit OERR = 0xFAB.1  // rx overflow error flag
#bit CREN = 0xFAB.4  // rx enable

// I/O PINS ~
#define STATUS_LED            PIN_A4
#define LCD_PIN               PIN_D1
#define MOTOR_PIN             PIN_D6
#define PIEZO_PIN             PIN_D0
#define SONAR_TRIGGER_PIN     PIN_B1
#define SONAR_ECHO_PIN        PIN_B2
#define STEER_PIN             PIN_B4
#define SENSOR_SERVO_PIN      PIN_B3

#define LEFT_IR_CONTROL_PIN   PIN_D4

#define RIGHT_IR_CONTROL_PIN  PIN_D7

#define LEFT_IR_DATA_PIN      PIN_D5

#define RIGHT_IR_DATA_PIN     PIN_B0
#define COMPASS_PULSE_PIN     PIN_C3
#define LEFT_BUMPER_PIN       PIN_D2
#define RIGHT_BUMPER_PIN      PIN_D3
#define WHEEL_PIN             PIN_C0
#define OMI_PIN               PIN_E0
#define GPS_PIN               PIN_E2


//#define TEST_A TRUE
#define TEST_A FALSE   //  testing OBSTICLE


#define ABS(a) ((a) >= 0 ? (a) : -(a))
#define MHZ 40
#define AND &&
#define SUCCESS 1
#define ERROR 0
#define X_FACTOR 1305  // meters per minute of longitude (Kanata Ont.)
#define Y_FACTOR 1848  // meters per minute of latutude (Kanata Ont.)
//#define X_FACTOR 1306  // meters per minute of longitude (Agonquin College, Ottawa, Ont.)
//#define Y_FACTOR 1852  // meters per minute of latutude (Agonquin College, Ottawa, Ont.)
#define NUMBER_OF_WAYPOINTS 6
#define ROUTE_MAX 15
#define SPEED_UPDATE_INTERVAL 100
#define GPS_UPDATE_INTERVAL 10000
#define TICKS_PER_METER 105

// states
#define _STANDBY   1
#define _BEGINNING 2
#define _MIDDLE    3
#define _ENDING    4
#define _OBSTACLE  5
#define _QUIT      6


// obsticle avoidance distances (in cm)
//#define TURN_START  180 + FRONT_OFFSET
#define TURN_START  200 + FRONT_OFFSET
#define CLOSE_TO_DESTINATION  3 // meters
#define TURN_END    TURN_START + 50
#define TOO_CLOSE    10 + FRONT_OFFSET


// FUNCTION LIST (aka prototypes) ~

// behavior
void standby(void);
void beginning(void);
void middle(void);
void ending(void);
void obstacle(void);
void quit(void);
void deadstop(void);
void handle_user_input(void);

// calculations
int8 gpsstring_to_minutes(void);  // (gpsstring --> inputlat & inpulon)
int16 xy_to_heading(signed int16, signed int16);
int16 xy_to_distance(signed int16, signed int16);
float sqrtb(float);
float atan_degree(float);

int16 get_speed(void);

// misc
 int1 go_meters(signed int16);
 void tx_info (void);


// GLOBAL VARIABLES  ~

// for navigation
//int16 distance=0;
int16 leg_length;
char input;              // input values for the menu
char str[80];
char gpsstring[100];
int8 route[ROUTE_MAX] = {0,1,0};
float current_lat;
float current_lon;
int8 state = _STANDBY ;
int16 desired_heading;

float lat[NUMBER_OF_WAYPOINTS] ;
float lon[NUMBER_OF_WAYPOINTS] ;
int8 current_leg = 1, target_waypoint;
int8 number_of_waypoints_in_route;
//int8 n;
int16 distance_remaining_since_gps_check;
int16 wheel_ticks_since_gps_check;
int1 time_to_report;  //~added april 27,2010

//  misc
signed int16 zone[6];    // 5 distance measurments from the sonar (cm) _\|/_

//  for interupts
int32 current_time=0;
int32 blink_time = 0;
int8 led_state;

/*
int16 steer_timer_value = 0 - (1500 * 10 / 4);
int16 sensor_servo_timer_value = 0 - (1500 * 10 / 4);
int16 servo_low_timer_value = 0 - (20000 /4 *10 );
*/
int16 steer_timer_value = 65536 - 3750;
int16 sensor_servo_timer_value = 65536 - 3750;
int16 servo_low_timer_value = 65536 - 50000;

int8 servo_state = 0;
int1 wheel_state = 0;
int16 wheel_count = 0;
int16 ms_since_update = 0;
const int16 update_interval = 200;
int16 wheel_count_recent = 0;
int16 wheel_count_old = 0;

// heartbeat
int32 hb_rate=72;  // (beats per minute)
int32 hb_on=80;    //
int32 hb_1=200;    //
int32 hb_2=1000;   //
int32 hb_state=0;  //

